@@ -1,6 +1,7 @@
 /*
-  
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -207,7 +208,7 @@ void ci_cork(ClientBase_T *s)
 void ci_uncork(ClientBase_T *s)
 {
 	int state;
-	TRACE(TRACE_DEBUG,"[%p] [%d] [%d], [%d]", s, s->rx, s->tx, s->timeout.tv_sec);
+	TRACE(TRACE_DEBUG,"[%p] [%d] [%d], [%ld]", s, s->rx, s->tx, s->timeout.tv_sec);
 
 	PLOCK(s->lock);
 	state = s->client_state;
