@@ -1,6 +1,8 @@
 /*
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -39,6 +41,42 @@ static GKeyFile *config_dict = NULL;
 static int configured = 0;
 
 
+long config_get_app_version(void)
+{
+	long version=0;
+	long temp=0;
+	char version_str_long[64];
+	sprintf(version_str_long,"%s",DM_VERSION);
+	TRACE(TRACE_INFO, "Version string expression [%s] => %s", version_str_long, DM_VERSION);
+	
+	GString *g_version_str_long=g_string_new(version_str_long);
+	GList *parts_version_str_long=g_string_split(g_version_str_long,"-");
+	GString *g_version_str=g_string_new(parts_version_str_long->data);
+	GList *parts_version_str=g_string_split(g_version_str,".");
+	
+	
+	temp=strtol((char *)parts_version_str->data, NULL, 10);
+	version=temp;
+	TRACE(TRACE_INFO, "\tVersion Part 1 [%ld] => final [%ld]", temp, version);
+	
+	parts_version_str = g_list_next(parts_version_str);
+	temp=strtol((char *)parts_version_str->data, NULL, 10);
+	version=version*10+temp;
+	TRACE(TRACE_INFO, "\tVersion Part 2 [%ld] => final [%ld]", temp, version);
+	
+	parts_version_str = g_list_next(parts_version_str);
+	temp=strtol((char *)parts_version_str->data, NULL, 10);
+	version=version*1000+temp;
+	TRACE(TRACE_INFO, "\tVersion Part 3 [%ld] => final [%ld]", temp, version);
+	
+	g_string_free(g_version_str_long,TRUE);
+	g_string_free(g_version_str,TRUE);
+	g_list_destroy(parts_version_str_long);
+	g_list_destroy(parts_version_str);
+		
+	return version;
+}
+
 void config_get_file(void)
 {
 	strncpy(configFile, DEFAULT_CONFIG_FILE, sizeof(configFile)-1);
@@ -97,7 +135,7 @@ int config_read(const char *config_filename)
 	if (stat(config_filename, &buf) == -1)
 		config_create(config_filename);
 
-        config_dict = g_key_file_new();
+	config_dict = g_key_file_new();
 	if (! g_key_file_load_from_file(config_dict, config_filename, G_KEY_FILE_NONE, NULL)) {
 		g_key_file_free(config_dict);
                 TRACE(TRACE_EMERG, "error reading config [%s]", config_filename);
