@@ -1,6 +1,8 @@
 /*
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -588,7 +590,7 @@ int db_noinferiors(uint64_t mailbox_idnr);
  */
 
 int db_append_msg(const char *msgdata, uint64_t mailbox_idnr, uint64_t user_idnr, 
-		char * internal_date, uint64_t * msg_idnr, gboolean recent);
+		const char * internal_date, uint64_t * msg_idnr, gboolean recent);
 
 /**
  * \brief move all messages from one mailbox to another.
