@@ -1,6 +1,7 @@
 /*
-  
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -836,7 +837,7 @@ static void _fetch_headers(ImapSession *self, body_fetch *bodyfetch, gboolean no
 			"AND status < %d "
 
 			//"AND n.headername %s IN ('%s') "	//old, from the sql point of view is slow, CC 2020
-			// "GROUP By m.message_idnr, n.headername, v.headervalue "
+			"GROUP By m.message_idnr, n.headername, v.headervalue "
 			// "having seq %s %d "
 			"ORDER BY m.message_idnr, seq",
 			not?"":fieldorder->str,
