@@ -1,5 +1,7 @@
 /*
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
@@ -132,16 +134,16 @@ static size_t dump_message_to_stream(DbmailMailbox *self, DbmailMessage *message
 
 	s = dbmail_message_to_string(message);
 
-	if (!strncmp(s, "From ", 5) == 0) {
-		ialist = internet_address_list_parse_string(g_mime_message_get_sender(GMIME_MESSAGE(message->content)));
-		sender = p_string_new(self->pool, "nobody@foo");
-		if (ialist) {
-			ia = internet_address_list_get_address(ialist, 0);
-			if (ia) {
-				char *addr = (char *) internet_address_mailbox_get_addr((InternetAddressMailbox *) ia);
-				g_strstrip(g_strdelimit(addr, "\"", ' '));
-				p_string_printf(sender, "%s", addr);
-			}
+	if (!strncmp(s, "From ", 5)) {
+		ialist = g_mime_message_get_sender(GMIME_MESSAGE(message->content));
+		if (internet_address_list_length(ialist) > 0) {
+			ia = internet_address_list_get_address (ialist, 0);
+			sender = p_string_new(self->pool, internet_address_to_string (ia, NULL, TRUE));
+			g_free(ia);
+			TRACE(TRACE_DEBUG, "Sender is: [%s]", (char *)sender);
+		} else {
+			TRACE(TRACE_DEBUG, "Setting sender to nobody@foo");
+			sender = p_string_new(self->pool, "nobody@foo");
 		}
 		g_object_unref(ialist);
 
@@ -393,9 +395,74 @@ char * dbmail_mailbox_orderedsubject(DbmailMailbox *self) {
 }
 
 /*
- * return self->ids as a string
+ * Returns imap modseq response for a user's mailbox
+ * 
+ * This function came from and is used with dbmail_mailbox_ids_as_string
  */
-char * dbmail_mailbox_ids_as_string(DbmailMailbox *self, gboolean uid, const char *sep) {
+char * dbmail_mailbox_imap_modseq_as_string(DbmailMailbox *self, gboolean uid) {
+	TRACE(TRACE_DEBUG, "Call: dbmail_mailbox_imap_modseq_as_string");
+	GString *t;
+	gchar *s = NULL;
+	GList *l = NULL;
+	GTree *msginfo;
+	GTree *msn;
+	uint64_t maxseq = 0;
+
+	if ((self->found == NULL) || g_tree_nnodes(self->found) <= 0) {
+		TRACE(TRACE_DEBUG, "no ids found");
+		return s;
+	}
+
+	t = g_string_new("");
+	if (uid || dbmail_mailbox_get_uid(self)) {
+		l = g_tree_keys(self->found);
+	} else {
+		l = g_tree_values(self->found);
+	}
+
+	msginfo = MailboxState_getMsginfo(self->mbstate);
+	msn = MailboxState_getMsn(self->mbstate);
+
+	while (l->data) {
+		uint64_t *key = (uint64_t *) l->data;
+		if (self->modseq) {
+			uint64_t *id;
+			if (uid || dbmail_mailbox_get_uid(self)) {
+				id = key;
+			} else {
+				id = g_tree_lookup(msn, key);
+			}
+
+			MessageInfo *info = g_tree_lookup(msginfo, id);
+			maxseq = max(maxseq, info->seq);
+		}
+		if (!g_list_next(l))
+			break;
+		l = g_list_next(l);
+	}
+
+	g_list_free(l);
+
+	if (self->modseq)
+		g_string_append_printf(t, " (MODSEQ %" PRIu64 ")", maxseq);
+
+	s = t->str;
+	g_string_free(t, FALSE);
+
+	return g_strchomp(s);
+
+}
+
+/*
+ * Returns users mailbox self->ids as a string concatenated with a seperator
+ * 
+ * Use with dbmail_mailbox_imap_modseq_as_string to generate imap response
+ */
+char * dbmail_mailbox_ids_as_string(
+		DbmailMailbox *self,
+		gboolean uid,
+		const char *sep
+	) {
 	TRACE(TRACE_DEBUG, "Call: dbmail_mailbox_ids_as_string");
 	GString *t;
 	gchar *s = NULL;
@@ -443,9 +510,6 @@ char * dbmail_mailbox_ids_as_string(DbmailMailbox *self, gboolean uid, const cha
 
 	g_list_free(h);
 
-	if (self->modseq)
-		g_string_append_printf(t, " (MODSEQ %" PRIu64 ")", maxseq);
-
 	s = t->str;
 	g_string_free(t, FALSE);
 
