@@ -1,6 +1,7 @@
 /*
-  
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -33,6 +34,8 @@ struct MailboxState_T {
 	uint64_t uidnext;
 	uint64_t owner_id;
 	uint64_t seq;
+	// the seq of the mailbox when the state is loaded
+	uint64_t state_seq;
 	// when loading is in differential mode, this flag will be incremented
 	unsigned differential_iterations;
 	//
