@@ -1,5 +1,7 @@
 /*
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
@@ -23,6 +25,11 @@
  */
 
 #include "dbmail.h"
+//#include "gmime/gmime-header.h"
+//#include "gmime/gmime-message.h"
+//#include "gmime/gmime-object.h"
+//#include "gmime/gmime-stream.h"
+//#include "gmime/gmime-stream-mem.h"
 
 extern DBParam_T db_params;
 #define DBPFX db_params.pfx
@@ -87,10 +94,10 @@ gchar * g_mime_object_get_body(const GMimeObject *object)
 	gchar *s = NULL, *b = NULL;
         unsigned i;
 	size_t l;
-	
+
 	g_return_val_if_fail(object != NULL, NULL);
 
-	s = g_mime_object_to_string(GMIME_OBJECT(object));
+	s = g_mime_object_to_string(GMIME_OBJECT(object), NULL);
 	assert(s);
 
 	i = find_end_of_header(s);
@@ -98,7 +105,7 @@ gchar * g_mime_object_get_body(const GMimeObject *object)
 		g_free(s);
 		return g_strdup("");
 	}
-	
+
 	b = s+i;
 	l = strlen(b);
 	memmove(s,b,l);
@@ -318,7 +325,6 @@ static char *find_type_header(const char *s)
 	}
 	h = header->str;
 	g_string_free(header,FALSE);
-	header = NULL;
 	return g_strstrip(h);
 }
 
@@ -328,7 +334,7 @@ static GMimeContentType *find_type(const char *s)
 	char *header = find_type_header(s);
 	if (! header)
 		return NULL;
-	type = g_mime_content_type_new_from_string(header);
+	type = g_mime_content_type_parse(NULL, header);
 	g_free(header);
 	return type;
 }
@@ -506,7 +512,7 @@ static gboolean store_mime_object(GMimeObject *parent, GMimeObject *object, Dbma
 static int store_head(GMimeObject *object, DbmailMessage *m)
 {
 	int r;
-	char *head = g_mime_object_get_headers(object);
+	char *head = g_mime_object_get_headers(object, NULL);
 	r = store_blob(m, head, 1);
 	g_free(head);
 	return r;
@@ -541,8 +547,8 @@ static gboolean store_mime_multipart(GMimeObject *object, DbmailMessage *m, cons
 	if (! skiphead && store_head(object,m) < 0) return TRUE;
 
 	boundary = g_mime_multipart_get_boundary((GMimeMultipart *)object);
-	preface = g_mime_multipart_get_preface((GMimeMultipart *)object);
-	postface = g_mime_multipart_get_postface((GMimeMultipart *)object);
+	preface = g_mime_multipart_get_prologue((GMimeMultipart *)object);
+	postface = g_mime_multipart_get_epilogue((GMimeMultipart *)object);
 
 	if (g_mime_content_type_is_type(GMIME_CONTENT_TYPE(content_type), "multipart", "*") &&
 			store_blob(m, preface, 0) < 0) return TRUE;
@@ -554,9 +560,12 @@ static gboolean store_mime_multipart(GMimeObject *object, DbmailMessage *m, cons
 	}
 
 	c = g_mime_multipart_get_count((GMimeMultipart *)object);
+    gboolean store_mime_object_result;
 	for (i=0; i<c; i++) {
 		GMimeObject *part = g_mime_multipart_get_part((GMimeMultipart *)object, i);
-		if (store_mime_object(object, part, m)) return TRUE;
+		// if (store_mime_object(object, part, m)) return TRUE;
+		store_mime_object_result = store_mime_object(object, part, m);
+		TRACE(TRACE_DEBUG,"store_mime_object: [%d] [%d]", i, store_mime_object_result);
 	}
 
 	if (boundary) {
@@ -610,7 +619,6 @@ gboolean store_mime_object(GMimeObject *parent, GMimeObject *object, DbmailMessa
 		// part of the rfc822 headers
 		skiphead = TRUE;
 
-		g_mime_header_list_set_stream (GMIME_MESSAGE(object)->mime_part->headers, NULL);
 		mime_part = g_mime_message_get_mime_part((GMimeMessage *)object);
 	} else
 		mime_part = object;
@@ -696,6 +704,9 @@ DbmailMessage * dbmail_message_new(Mempool_T pool)
 	self->header_dict = g_hash_table_new_full((GHashFunc)g_str_hash,
 			(GEqualFunc)g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
 	
+	/* set the charset */
+	self->charset = "utf-8";
+
 	dbmail_message_set_class(self, DBMAIL_MESSAGE);
 	
 	return self;
@@ -709,7 +720,6 @@ void dbmail_message_free(DbmailMessage *self)
 		return;
 
 	if (self->content) {
-		g_object_unref(self->content);
 		self->content = NULL;
 	}
 
@@ -805,7 +815,7 @@ DbmailMessage * dbmail_message_init_with_string(DbmailMessage *self, const char
 	parser = g_mime_parser_new_with_stream(self->stream);
 
 
-	content = GMIME_OBJECT(g_mime_parser_construct_message(parser));
+	content = GMIME_OBJECT(g_mime_parser_construct_message(parser, NULL));
 	if (content) {
 		g_object_unref(parser);
 		dbmail_message_set_class(self, DBMAIL_MESSAGE);
@@ -813,7 +823,7 @@ DbmailMessage * dbmail_message_init_with_string(DbmailMessage *self, const char
 		if (from[0])
 			dbmail_message_set_internal_date(self, from);
 	} else {
-		content = GMIME_OBJECT(g_mime_parser_construct_part(parser));
+		content = GMIME_OBJECT(g_mime_parser_construct_part(parser, NULL));
 		g_object_unref(parser);
 		if (content) {
 			dbmail_message_set_class(self, DBMAIL_MESSAGE_PART);
@@ -840,14 +850,14 @@ uint64_t dbmail_message_get_physid(const DbmailMessage *self)
 	return self->id;
 }
 
-void dbmail_message_set_internal_date(DbmailMessage *self, char *internal_date)
+void dbmail_message_set_internal_date(DbmailMessage *self, const char *internal_date)
 {
 	self->internal_date = time(NULL);
 	if (internal_date && strlen(internal_date)) {
 		time_t dt;
-	        if ((dt = g_mime_utils_header_decode_date(
-						internal_date,
-						&(self->internal_date_gmtoff)))) {
+		GDateTime * gdt;
+		if ((gdt = g_mime_utils_header_decode_date(internal_date))) {
+			dt = g_date_time_to_unix(gdt);
 			self->internal_date = dt;
 		}
 		TRACE(TRACE_DEBUG, "internal_date [%s] [%ld] offset [%d]",
@@ -892,7 +902,7 @@ const char * dbmail_message_get_envelope_recipient(const DbmailMessage *self)
 
 void dbmail_message_set_header(DbmailMessage *self, const char *header, const char *value)
 {
-	g_mime_object_prepend_header(GMIME_OBJECT(self->content), header, value);
+	g_mime_object_prepend_header(GMIME_OBJECT(self->content), header, value, self->charset);
 }
 
 const gchar * dbmail_message_get_header(const DbmailMessage *self, const char *header)
@@ -918,13 +928,23 @@ GList * dbmail_message_get_header_repeated(const DbmailMessage *self, const char
 {
 	GMimeHeaderList *headers = g_mime_object_get_header_list(
 			GMIME_OBJECT(self->content));
-       
+
+	GMimeHeader *header_item;
 	struct payload data;
 	memset(&data, 0, sizeof(struct payload));
 	data.header = header;
 	data.list = NULL;
+	guint i;
+	guint headers_qty = g_mime_header_list_get_count(headers);
 
-	g_mime_header_list_foreach(headers, _get_header_repeated, &data);
+	for (i = 0; i < headers_qty; i++) {
+		header_item = g_mime_header_list_get_header_at (headers, (int) i);
+		(*_get_header_repeated) (
+			g_mime_header_get_name(header_item),
+			g_mime_header_get_value(header_item),
+			&data
+		);
+	}
 
 	return data.list;
 }
@@ -947,7 +967,7 @@ GList * dbmail_message_get_header_addresses(DbmailMessage *message, const char *
 	
 	TRACE(TRACE_INFO, "mail address parser looking at field [%s] with value [%s]", field_name, field_value);
 	
-	if ((ialist = internet_address_list_parse_string(field_value)) == NULL) {
+	if ((ialist = internet_address_list_parse(NULL, field_value)) == NULL) {
 		TRACE(TRACE_NOTICE, "mail address parser error parsing header field");
 		return NULL;
 	}
@@ -980,7 +1000,7 @@ const char * dbmail_message_get_charset(DbmailMessage *self)
 gchar * dbmail_message_to_string(const DbmailMessage *self) 
 {
 	assert(self && self->content);
-	return g_mime_object_to_string(GMIME_OBJECT(self->content));
+	return g_mime_object_to_string(GMIME_OBJECT(self->content), NULL);
 }
 gchar * dbmail_message_body_to_string(const DbmailMessage *self)
 {
@@ -1311,22 +1331,20 @@ int _message_insert(DbmailMessage *self,
 void _message_cache_envelope_date(const DbmailMessage *self)
 {
 	time_t date = self->internal_date;
+	GDateTime* gdate;
 	char *value;
 	char datefield[CACHE_WIDTH];
 	char sortfield[CACHE_WIDTH];
 	uint64_t headervalue_id = 0;
 	uint64_t headername_id = 0;
 
-	value = g_mime_utils_header_format_date(
-			self->internal_date, 
-			self->internal_date_gmtoff);
+	gdate = g_date_time_new_from_unix_local(self->internal_date);
+	value = g_mime_utils_header_format_date(gdate);
+	gdate = NULL;
 
 	memset(sortfield, 0, sizeof(sortfield));
 	strftime(sortfield, CACHE_WIDTH-1, "%Y-%m-%d %H:%M:%S", gmtime(&date));
 
-	if (self->internal_date_gmtoff)
-		date += (self->internal_date_gmtoff * 36);
-
 	memset(datefield, 0, sizeof(datefield));
 	strftime(datefield, 20, "%Y-%m-%d", gmtime(&date));
 
@@ -1347,38 +1365,45 @@ int dbmail_message_cache_headers(const DbmailMessage *self)
 	GMimeObject *part;
 	GMimeContentType *content_type;
 	GMimeContentDisposition *content_disp;
+	const char *header_name, *header_raw_value;
 
 	if (! GMIME_IS_MESSAGE(self->content)) {
 		TRACE(TRACE_ERR,"self->content is not a message");
 		return -1;
 	}
 
-	/* 
+	/*
 	 * store all headers as-is, plus separate copies for
 	 * searching and sorting
-	 * 
+	 *
 	 * */
 	GMimeHeaderList *headers = g_mime_object_get_header_list(
-			GMIME_OBJECT(self->content));
-	g_mime_header_list_foreach(headers, (GMimeHeaderForeachFunc)_header_cache,
-			(gpointer)self);
+		GMIME_OBJECT(self->content));
+	int header_count = g_mime_header_list_get_count(headers);
+	int i;
+	for (i=0; i<header_count; i++) {
+		GMimeHeader *header = g_mime_header_list_get_header_at((GMimeHeaderList*)headers, i);
 
-	/* 
+		header_name = g_mime_header_get_name (header);
+		header_raw_value = g_mime_header_get_raw_value (header);
+		_header_cache(header_name, header_raw_value, (gpointer)self);
+	}
+
+	/*
 	 * gmime treats content-type and content-disposition differently
 	 *
 	 */
 	part = g_mime_message_get_mime_part(GMIME_MESSAGE(self->content));
 	if ((content_type = g_mime_object_get_content_type(part))) {
-		char *value = g_mime_content_type_to_string(content_type);
+		char *value = g_mime_content_type_get_mime_type(content_type);
 		_header_cache("content-type", (const char *)value, (gpointer)self);
-		free(value);
+		g_free(value);
 	}
 
 	if ((content_disp = g_mime_object_get_content_disposition(part))) {
-		char *value = g_mime_content_disposition_to_string(
-				content_disp, FALSE);
+		char *value = g_mime_content_disposition_encode(content_disp, NULL);
 		_header_cache("content-disposition", (const char *)value, (gpointer)self);
-		free(value);
+		g_free(value);
 	}
 
 	/* 
@@ -1402,7 +1427,6 @@ int dbmail_message_cache_headers(const DbmailMessage *self)
 static int _header_name_get_id(const DbmailMessage *self, const char *header, uint64_t *id)
 {
 	uint64_t *tmp = NULL;
-	gpointer cacheid;
 	gchar *case_header, *safe_header, *frag;
 	Connection_T c; ResultSet_T r; PreparedStatement_T s;
 	Field_T config;
@@ -1411,11 +1435,6 @@ static int _header_name_get_id(const DbmailMessage *self, const char *header, ui
 
 	// rfc822 headernames are case-insensitive
 	safe_header = g_ascii_strdown(header,-1);
-	if ((cacheid = g_hash_table_lookup(self->header_dict, (gconstpointer)safe_header)) != NULL) {
-		*id = *(uint64_t *)cacheid;
-		g_free(safe_header);
-		return 1;
-	}
 
 	config_get_value("header_cache_readonly", "DBMAIL", config);
 	if (strlen(config)) {
@@ -1479,6 +1498,7 @@ static int _header_name_get_id(const DbmailMessage *self, const char *header, ui
 	}
 
 	*id = *tmp;
+	TRACE(TRACE_DEBUG,"Adding cache: [%s] [%lu]", safe_header, *tmp);
 	g_hash_table_insert(self->header_dict, (gpointer)(safe_header), (gpointer)(tmp));
 	return 1;
 }
@@ -1580,6 +1600,7 @@ static gboolean _header_insert(uint64_t physmessage_id, uint64_t headername_id,
 
 	Connection_T c; PreparedStatement_T s; volatile gboolean t = TRUE;
 
+	TRACE(TRACE_DEBUG, "Inserting header: [%lu] [%lu] [%lu]", physmessage_id, headername_id, headervalue_id);
 	c = db_con_get();
 	db_con_clear(c);
 	TRY
@@ -1591,6 +1612,7 @@ static gboolean _header_insert(uint64_t physmessage_id, uint64_t headername_id,
 		db_stmt_exec(s);
 		db_commit_transaction(c);
 	CATCH(SQLException)
+		LOG_SQLERROR;
 		db_rollback_transaction(c);
 		t = FALSE;
 	FINALLY
@@ -1659,7 +1681,8 @@ static void _header_cache(const char *header, const char *raw, gpointer user_dat
 	uint64_t headername_id = 0;
 	uint64_t headervalue_id;
 	DbmailMessage *self = (DbmailMessage *)user_data;
-	time_t date;
+	GDateTime *date;
+	gchar* date_fmt;
 	volatile gboolean isaddr = 0, isdate = 0, issubject = 0;
 	const char *charset = dbmail_message_get_charset(self);
 	char datefield[CACHE_WIDTH];
@@ -1698,8 +1721,13 @@ static void _header_cache(const char *header, const char *raw, gpointer user_dat
 	else if (g_ascii_strcasecmp(header,"Date")==0)
 		isdate=1;
 
-
 	value = dbmail_iconv_decode_field(raw, charset, isaddr);
+	value = g_strstrip(value);
+
+	TRACE(TRACE_DEBUG,
+		"headername [%s] id [%lu] raw [%s] value [%s] isaddr [%d] issubject [%d] isdate [%d]",
+		header, headername_id, raw, value, isaddr, issubject, isdate
+	);
 
 	if ((! value) || (strlen(value) == 0)) {
 		if (value) 
@@ -1711,7 +1739,7 @@ static void _header_cache(const char *header, const char *raw, gpointer user_dat
 	if(isaddr) {
 		GString *store;
 		int i,j=0;
-		emaillist = internet_address_list_parse_string(value);
+		emaillist = internet_address_list_parse(NULL, value);
 		store = _header_addresses(emaillist);
 
 		i = internet_address_list_length(emaillist);
@@ -1752,14 +1780,18 @@ static void _header_cache(const char *header, const char *raw, gpointer user_dat
 
 	memset(datefield, 0, sizeof(datefield));
 	if(isdate) {
-		int offset;
-		date = g_mime_utils_header_decode_date(value,&offset);
-		strftime(sortfield, CACHE_WIDTH-1, "%Y-%m-%d %H:%M:%S", gmtime(&date));
-
-		date += (offset * 36); // +0200 -> offset 200
-		strftime(datefield, 20,"%Y-%m-%d", gmtime(&date));
-		TRACE(TRACE_DEBUG,"Date is [%s] offset [%d], datefield [%s]",
-				value, offset, datefield);
+		date = g_mime_utils_header_decode_date(value);
+		date_fmt = g_date_time_format(date, "%Y-%m-%d %H:%M:%S");
+		TRACE(TRACE_DEBUG,"date_fmt [%s]", date_fmt);
+		if (date_fmt) {
+			g_utf8_strncpy(sortfield, date_fmt, CACHE_WIDTH-1);
+			TRACE(TRACE_DEBUG,"sortfield [%s]", sortfield);
+			date_fmt = g_date_time_format(date, "%Y-%m-%d");
+			g_utf8_strncpy(datefield, date_fmt, CACHE_WIDTH-1);
+			TRACE(TRACE_DEBUG,"Date is [%s] datefield [%s]", value, datefield);
+			g_date_time_unref(date);
+			g_free(date_fmt);
+		}
 	}
 
 	if (sortfield[0] == '\0')
@@ -1779,7 +1811,6 @@ static void _header_cache(const char *header, const char *raw, gpointer user_dat
 	headervalue_id=0;
 
 	emaillist=NULL;
-	date=0;
 }
 
 static void insert_field_cache(uint64_t physid, const char *field, const char *value)
@@ -1810,6 +1841,7 @@ static void insert_field_cache(uint64_t physid, const char *field, const char *v
 	g_free(clean_value);
 }
 
+
 #define DM_ADDRESS_TYPE_TO "To"
 #define DM_ADDRESS_TYPE_CC "Cc"
 #define DM_ADDRESS_TYPE_FROM "From"
@@ -1827,31 +1859,34 @@ void dbmail_message_cache_referencesfield(const DbmailMessage *self)
 
 	// Some clients will put parent in the in-reply-to header only and the grandparents and older in references
 	field = g_strconcat(referencesfield, " ", inreplytofield, NULL);
-	refs = g_mime_references_decode(field);
+	refs = g_mime_references_parse(NULL, field);
 	g_free(field);
 
 	if (! refs) {
 		TRACE(TRACE_DEBUG, "reference_decode failed [%" PRIu64 "]", self->id);
 		return;
 	}
-	
+
 	head = refs;
 	tree = g_tree_new_full((GCompareDataFunc)dm_strcmpdata, NULL, NULL, NULL);
-	
-	while (refs->msgid) {
-		if (! g_tree_lookup(tree,refs->msgid)) {
-			insert_field_cache(self->id, "references", refs->msgid);
-			g_tree_insert(tree,refs->msgid,refs->msgid);
+
+	int refs_len = g_mime_references_length(refs);
+	int i;
+	const char* msgid;
+
+	for (i = 0; i < refs_len; ++i) {
+		msgid = g_mime_references_get_message_id (refs, i);
+
+		if (! g_tree_lookup(tree, (gconstpointer) msgid)) {
+			insert_field_cache(self->id, "references", msgid);
+			g_tree_insert(tree, (char *)msgid, (char *)msgid);
 		}
-		if (refs->next == NULL)
-			break;
-		refs = refs->next;
 	}
 
 	g_tree_destroy(tree);
-	g_mime_references_clear(&head);
+	g_mime_references_clear(head);
 }
-	
+
 void dbmail_message_cache_envelope(const DbmailMessage *self)
 {
 	char *envelope = NULL;
@@ -1912,9 +1947,9 @@ DbmailMessage * dbmail_message_construct(DbmailMessage *self,
 
 	// set basic headers
 	TRACE(TRACE_DEBUG, "from: [%s] to: [%s] subject: [%s] body: [%s]", from, to, subject, body);
-	g_mime_message_set_sender(message, from);
-	g_mime_message_set_subject(message, subject);
-	g_mime_message_add_recipient(message, GMIME_RECIPIENT_TYPE_TO, NULL, to);
+	g_mime_message_add_mailbox(message, GMIME_ADDRESS_TYPE_SENDER, NULL, from);
+	g_mime_message_set_subject(message, subject, self->charset);
+	g_mime_message_add_mailbox(message, GMIME_ADDRESS_TYPE_TO, NULL, to);
 
 	// construct mime-part
 	mime_part = g_mime_part_new();
@@ -1943,7 +1978,7 @@ DbmailMessage * dbmail_message_construct(DbmailMessage *self,
 	g_object_unref(fstream);
 
 	content = g_mime_data_wrapper_new_with_stream(stream, encoding);
-	g_mime_part_set_content_object(mime_part, content);
+	g_mime_part_set_content(mime_part, content);
 	g_object_unref(content);
 	
 	// Content-Type
@@ -1957,13 +1992,25 @@ DbmailMessage * dbmail_message_construct(DbmailMessage *self,
 	// Content-Transfer-Encoding
 	switch(encoding) {
 		case GMIME_CONTENT_ENCODING_BASE64:
-			g_mime_object_set_header(GMIME_OBJECT(mime_part),"Content-Transfer-Encoding", "base64");
+			g_mime_object_set_header(
+				GMIME_OBJECT(mime_part),
+				"Content-Transfer-Encoding",
+				"base64",
+				self->charset);
 			break;
 		case GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE:
-			g_mime_object_set_header(GMIME_OBJECT(mime_part),"Content-Transfer-Encoding", "quoted-printable");
+			g_mime_object_set_header(
+				GMIME_OBJECT(mime_part),
+				"Content-Transfer-Encoding",
+				"quoted-printable",
+				self->charset);
 			break;
 		default:
-			g_mime_object_set_header(GMIME_OBJECT(mime_part),"Content-Transfer-Encoding", "7bit");
+			g_mime_object_set_header(
+				GMIME_OBJECT(mime_part),
+				"Content-Transfer-Encoding",
+				"7bit",
+				self->charset);
 			break;
 	}
 
@@ -2229,7 +2276,7 @@ static int parse_and_escape(const char *in, char **out)
 	const char *addr;
 
 	TRACE(TRACE_DEBUG, "parsing address [%s]", in);
-	ialist = internet_address_list_parse_string(in);
+	ialist = internet_address_list_parse(NULL, in);
 	if (!ialist) {
                 TRACE(TRACE_NOTICE, "unable to parse email address [%s]", in);
                 return -1;
@@ -2464,7 +2511,7 @@ static int send_reply(DbmailMessage *message, const char *body, GList *aliases)
 
 	usubject = dbmail_iconv_decode_text(subject);
        	unewsubject = g_strconcat("Re: ", usubject, NULL);
-	newsubject = g_mime_utils_header_encode_text(unewsubject);
+	newsubject = g_mime_utils_header_encode_text(NULL, unewsubject, message->charset);
 	g_free(usubject);
 	g_free(unewsubject);
 
@@ -2709,7 +2756,6 @@ int insert_messages(DbmailMessage *message, List_T dsnusers)
 	else
 		TRACE(TRACE_INFO, "Using default hard bounce for quota failure");
 
-
 	tmpid = message->msg_idnr; // for later removal
 
 	// TODO: Run a Sieve script associated with the internal delivery user.
@@ -2813,6 +2859,9 @@ int insert_messages(DbmailMessage *message, List_T dsnusers)
 				TRACE(TRACE_NOTICE, "forwaring failed, reporting transient error.");
 				set_dsn(&delivery->dsn, DSN_CLASS_TEMP, 1, 1);
 			}
+			if (from) {
+				g_free((char *)from);
+			}
 		}
 		if (! p_list_next(dsnusers))
 			break;
