@@ -1,6 +1,8 @@
 /*
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -1457,6 +1459,8 @@ char *dbmail_imap_astring_as_string(const char *s)
 
 	if (! s)
 		return g_strdup("\"\"");
+	if (! strlen(s))
+		return g_strdup("\"\"");
 
 	l = g_strdup(s);
 	t = l;
@@ -1577,7 +1581,7 @@ static GList * imap_append_hash_as_string(GList *list, const char *type)
 			clean2 = g_strcompress(clean1);
 
 			if (g_mime_utils_text_is_8bit((const unsigned char *)clean2, strlen(clean2))) {
-				clean1 = g_mime_utils_header_encode_text(clean2);
+				clean1 = g_mime_utils_header_encode_text(NULL, clean2, NULL);
 				g_free(clean2);
 				clean2 = clean1;
 			}
@@ -1620,7 +1624,7 @@ static GList * imap_append_disposition_as_string(GList *list, GMimeObject *part)
 	const char *disp = g_mime_object_get_header(part, "Content-Disposition");
 	
 	if(disp) {
-		disposition = g_mime_content_disposition_new_from_string(disp);
+		disposition = g_mime_content_disposition_parse(NULL, disp);
 		t = g_list_append_printf(t,"\"%s\"",
 				g_mime_content_disposition_get_disposition(disposition));
 		
@@ -1960,7 +1964,7 @@ GList* dbmail_imap_append_alist_as_plist(GList *list, InternetAddressList *ialis
 
 			/* personal name */
 			if (name) {
-				char * encname = g_mime_utils_header_encode_phrase(name);
+				char * encname = g_mime_utils_header_encode_phrase(NULL, name, NULL);
 				g_strdelimit(encname,"\"\\",' ');
 				g_strstrip(encname);
 				s = dbmail_imap_astring_as_string(encname);
@@ -2085,10 +2089,10 @@ static GList * envelope_address_part(GList *list, GMimeMessage *message, const c
 	if (result) {
 		result_enc = dbmail_iconv_str_to_utf8(result, charset);
 		t = imap_cleanup_address(result_enc);
-	       	g_free(result_enc);
-		alist = internet_address_list_parse_string(t);
+		g_free(result_enc);
+		alist = internet_address_list_parse(NULL, t);
 		g_free(t);
-		list = dbmail_imap_append_alist_as_plist(list, (InternetAddressList *)alist);
+		list = dbmail_imap_append_alist_as_plist(list, alist);
 		g_object_unref(alist);
 		alist = NULL;
 	} else {
@@ -2133,7 +2137,7 @@ char * imap_get_envelope(GMimeMessage *message)
 {
 	GMimeObject *part;
 	GList *list = NULL;
-	char *result;
+	const char *result;
 	char *s = NULL, *t = NULL;
 	const char *h;
 
@@ -2145,26 +2149,24 @@ char * imap_get_envelope(GMimeMessage *message)
 	
 	part = GMIME_OBJECT(message);
 	/* date */
-	result = g_mime_message_get_date_as_string(message);
+	result = g_mime_object_get_header(part, "Date");
 	if (result) {
 		t = dbmail_imap_astring_as_string(result);
 		list = g_list_append_printf(list,"%s", t);
-		g_free(result);
 		g_free(t);
-		result = NULL;
 	} else {
 		list = g_list_append_printf(list,"NIL");
 	}
 	
 	/* subject */
-	result = (char *)g_mime_object_get_header(GMIME_OBJECT(message),"Subject");
+	result = g_mime_object_get_header(GMIME_OBJECT(message),"Subject");
 
 	if (result) {
 		const char *charset = message_get_charset(message);
 		char * subj = dbmail_iconv_str_to_utf8(result, charset);
 		TRACE(TRACE_DEBUG, "[%s] [%s] -> [%s]", charset, result, subj);
 		if (g_mime_utils_text_is_8bit((unsigned char *)subj, strlen(subj))) {
-			s = g_mime_utils_header_encode_text((const char *)subj);
+			s = g_mime_utils_header_encode_text(NULL, (const char *)subj, NULL);
 			TRACE(TRACE_DEBUG, "[%s] -> [%s]", subj, s);
 			g_free(subj);
 			subj = s;
@@ -2204,21 +2206,29 @@ char * imap_get_envelope(GMimeMessage *message)
 	/* in-reply-to */
 	list = imap_append_header_as_string(list,part,"In-Reply-to");
 	/* message-id */
-	result = (char *)g_mime_message_get_message_id(message);
+	result = g_mime_message_get_message_id(message);
 	if (result && (! g_strrstr(result,"=")) && (! g_strrstr(result,"@(none)"))) {
                 t = g_strdup_printf("<%s>", result);
 		s = dbmail_imap_astring_as_string(t);
 		list = g_list_append_printf(list,"%s", s);
 		g_free(s);
-                g_free(t);
+		g_free(t);
+		g_free((char *)result);
 	} else {
 		list = g_list_append_printf(list,"NIL");
 	}
 
 	s = dbmail_imap_plist_as_string(list);
 
+	GList * element;
+	list = g_list_first(list);
+	while ((element = g_list_next(list))) {
+		g_free(element->data);
+		list = g_list_next(list);
+	}
+
 	g_list_destroy(list);
-	
+
 	return s;
 }
 
@@ -2245,14 +2255,14 @@ char * imap_get_logical_part(const GMimeObject *object, const char * specifier)
 	}
 
 	if (MATCH(specifier,"HEADER") || MATCH(specifier,"MIME")) {
-		t = g_mime_object_get_headers(GMIME_OBJECT(object));
+		t = g_mime_object_get_headers(GMIME_OBJECT(object), NULL);
 		s = get_crlf_encoded(t);
 		g_free(t);
 		s = g_realloc(s, strlen(s) + 3);
 		strcat(s, "\r\n");
 	} else {
 		if (rfc822)
-			t = g_mime_object_to_string(GMIME_OBJECT(object));
+			t = g_mime_object_to_string(GMIME_OBJECT(object), NULL);
 		else
 			t = g_mime_object_get_body(GMIME_OBJECT(object));
 		s = get_crlf_encoded(t);
