@@ -1,6 +1,8 @@
 /*
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -1704,7 +1706,7 @@ void _ic_append_enter(dm_thread_data *D)
 	
 	message = p_string_str(self->args[i]);
 
-	D->status = db_append_msg(message, mboxid, self->userid, (char *)internal_date, &message_id, recent);
+	D->status = db_append_msg(message, mboxid, self->userid, internal_date, &message_id, recent);
 
 	switch (D->status) {
 	case -1:
@@ -1925,6 +1927,7 @@ static void sorted_search_enter(dm_thread_data *D)
 	DbmailMailbox *mb;
 	int result = 0;
 	gchar *s = NULL;
+	gchar *imap_modseq = NULL;
 	const gchar *cmd;
 
 	search_order order = self->order;
@@ -1972,6 +1975,9 @@ static void sorted_search_enter(dm_thread_data *D)
 			break;
 			case SEARCH_UNORDERED:
 				s = dbmail_mailbox_ids_as_string(mb, FALSE, " ");
+				imap_modseq = dbmail_mailbox_imap_modseq_as_string(mb, FALSE);
+				s = g_strconcat(s, imap_modseq, NULL);
+				g_free(imap_modseq);
 			break;
 			case SEARCH_THREAD_ORDEREDSUBJECT:
 				s = dbmail_mailbox_orderedsubject(mb);
@@ -2383,6 +2389,8 @@ static void _ic_store_enter(dm_thread_data *D)
 	self->cmd = &cmd;
 
 	if ( update ) {
+		/* flags need and seq update of the mailbox in order to trigger updates to other connected clients */
+		db_mailbox_seq_update(MailboxState_getId(self->mailbox->mbstate), 0);
 		char *flags = MailboxState_flags(self->mailbox->mbstate);
 		dbmail_imap_session_buff_printf(self, "* FLAGS (%s)\r\n", flags);
 		dbmail_imap_session_buff_printf(self, "* OK [PERMANENTFLAGS (%s \\*)] Flags allowed.\r\n", flags);
