@@ -1,6 +1,8 @@
 /*
  Copyright (c) 2002 Aaron Stone, aaron@serendipity.cx
  Copyright (c) 2004-2010 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -1344,7 +1346,7 @@ GList * auth_get_user_aliases(uint64_t user_idnr)
 	GList *entlist, *fldlist, *attlist;
 	
 	g_string_printf(t,"%s=%" PRIu64 "", _ldap_cfg.field_nid, user_idnr);
-	if ((entlist = __auth_get_every_match(t->str, fields))) {
+	if ((entlist = __auth_get_every_match(t->str, (const char **)fields))) {
 		entlist = g_list_first(entlist);
 		fldlist = g_list_first(entlist->data);
 		attlist = g_list_first(fldlist->data);
