@@ -1,6 +1,8 @@
 /* 
-
  Copyright (C) 1999-2004 Aaron Stone aaron at serendipity dot cx
+ Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -434,8 +436,8 @@ int sort_errparse(sieve2_context_t *s, void *my)
 	lineno = sieve2_getvalue_int(s, "lineno");
 	message = sieve2_getvalue_string(s, "message");
 
-	TRACE(TRACE_INFO, "Error is PARSE: Line is [%d], Message is [%s]", lineno, message);
-
+	TRACE(TRACE_ERR, "SIEVE Error is PARSE: Line is [%d], Message is [%s]", lineno, message);
+	/*
 	g_string_append_printf(m->result->errormsg, "Parse error on line [%d]: %s", lineno, message);
 
 	if (m->message) {
@@ -448,7 +450,7 @@ int sort_errparse(sieve2_context_t *s, void *my)
 		send_alert(m->user_idnr, "Sieve script parse error", alertbody);
 		g_free(alertbody);
 	}
-
+	*/
 	m->result->error_parse = 1;
 	return SIEVE2_OK;
 }
@@ -460,7 +462,7 @@ int sort_errexec(sieve2_context_t *s, void *my)
 
 	message = sieve2_getvalue_string(s, "message");
 
-	TRACE(TRACE_INFO, "Error is EXEC: Message is [%s]", message);
+	TRACE(TRACE_ERR, "SIEVE Error is EXEC: Message is [%s]", message);
 
 	/* This turns out to be incredibly annoying, as libSieve
 	 * throws execution errors on malformed addresses coming
@@ -513,6 +515,7 @@ int sort_getscript(sieve2_context_t *s, void *my)
 			return SIEVE2_ERROR_FAIL;
 		}
 		sieve2_setvalue_string(s, "script", m->s_buf);
+		TRACE(TRACE_INFO, "Script\n%s", m->s_buf);
 	} else {
 		return SIEVE2_ERROR_BADARGS;
 	}
@@ -523,19 +526,19 @@ int sort_getscript(sieve2_context_t *s, void *my)
 int sort_getheader(sieve2_context_t *s, void *my)
 {
 	struct sort_context *m = (struct sort_context *)my;
-	char *header;
+	const char *header;
 	char **bodylist;
 	GList *headers;
 	unsigned i;
 
-	header = (char *)sieve2_getvalue_string(s, "header");
+	header = sieve2_getvalue_string(s, "header");
 	
 	headers = dbmail_message_get_header_repeated(m->message, header);
 	
 	bodylist = g_new0(char *,g_list_length(headers)+1);
 	i = 0;
 	while (headers) {
-		char *decoded = dbmail_iconv_decode_text((char *)headers->data);
+		char *decoded = dbmail_iconv_decode_text(headers->data);
 		bodylist[i++] = decoded;
 		/* queue the decoded value for freeing later on */
 		m->freelist = g_list_prepend(m->freelist, decoded);
@@ -547,7 +550,9 @@ int sort_getheader(sieve2_context_t *s, void *my)
 	g_list_free(g_list_first(headers));
 
 	/* We have to free the header array. */
-	m->freelist = g_list_prepend(m->freelist, bodylist);
+	if (m->freelist) {
+		m->freelist = g_list_prepend(m->freelist, bodylist);
+	}
 
 	for (i = 0; bodylist[i] != NULL; i++) {
 		TRACE(TRACE_INFO, "Getting header [%s] returning value [%s]",
@@ -705,9 +710,7 @@ static int sort_teardown(sieve2_context_t **s2c,
 	assert(s2c != NULL);
 	assert(sc != NULL);
 
-	sieve2_context_t *sieve2_context = *s2c;
 	struct sort_context *sort_context = *sc;
-	int res;
 
 	g_list_destroy(sort_context->freelist);
 
@@ -715,13 +718,6 @@ static int sort_teardown(sieve2_context_t **s2c,
 		g_free(sort_context);
 	}
 
-	res = sieve2_free(&sieve2_context);
-	if (res != SIEVE2_OK) {
-		TRACE(TRACE_ERR, "Error [%d] when calling sieve2_free: [%s]",
-			res, sieve2_errstr(res));
-		return DM_EGENERAL;
-	}
-
 	*s2c = NULL;
 	*sc = NULL;
 
