@@ -1,7 +1,8 @@
 /*
-  
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
- Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -966,14 +967,13 @@ int server_mainloop(ServerConfig_T *config, const char *servicename)
 {
 	strncpy(config->process_name, servicename, FIELDSIZE-1);
 
-	g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
+	g_mime_init();
 	g_mime_parser_get_type();
 	g_mime_stream_get_type();
 	g_mime_stream_mem_get_type();
 	g_mime_stream_file_get_type();
 	g_mime_stream_buffer_get_type();
 	g_mime_stream_filter_get_type();
-	g_mime_filter_crlf_get_type();
 
 	tls_context = tls_init();
 
