@@ -1,5 +1,7 @@
 /*
- *  Copyright (c) 2005-2012 NFG Net Facilities Group BV support@nfg.nl
+ *   Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ *   Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ *   Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public License
@@ -797,7 +799,7 @@ START_TEST(test_g_tree_merge_not)
 	}
 
 	g_tree_merge(a,b,IST_SUBSEARCH_NOT);
-	fail_unless(g_tree_nnodes(a)==5002,"g_tree_merge failed. Too few nodes in a. [%ld]", g_tree_nnodes(a));
+	fail_unless(g_tree_nnodes(a)==5002,"g_tree_merge failed. Too few nodes in a. [%d]", g_tree_nnodes(a));
 	
 	g_tree_destroy(a);
 	g_tree_destroy(b);
@@ -810,7 +812,7 @@ START_TEST(test_g_tree_merge_not)
 		tree_add_key(a, r);
 
 	g_tree_merge(a,b,IST_SUBSEARCH_NOT);
-	fail_unless(g_tree_nnodes(a)==5002,"g_tree_merge failed. Too few nodes in a. [%ld]", g_tree_nnodes(a));
+	fail_unless(g_tree_nnodes(a)==5002,"g_tree_merge failed. Too few nodes in a. [%d]", g_tree_nnodes(a));
 	
 	g_tree_destroy(a);
 	g_tree_destroy(b);
@@ -831,7 +833,7 @@ START_TEST(test_g_tree_merge_or)
 		tree_add_key(b, r);
 
 	g_tree_merge(a,b,IST_SUBSEARCH_OR);
-	fail_unless(g_tree_nnodes(a)==45002,"g_tree_merge failed. Too many nodes in a. [%ld]", g_tree_nnodes(a));
+	fail_unless(g_tree_nnodes(a)==45002,"g_tree_merge failed. Too many nodes in a. [%d]", g_tree_nnodes(a));
 	
 	g_tree_destroy(a);
 	g_tree_destroy(b);
@@ -858,7 +860,7 @@ START_TEST(test_g_tree_merge_and)
 	}
 
 	g_tree_merge(a,b,IST_SUBSEARCH_AND);
-	fail_unless(g_tree_nnodes(a)==9001,"g_tree_merge failed. Too few nodes in a.[%ld]", g_tree_nnodes(a));
+	fail_unless(g_tree_nnodes(a)==9001,"g_tree_merge failed. Too few nodes in a.[%d]", g_tree_nnodes(a));
 	
 	g_tree_destroy(a);
 	g_tree_destroy(b);
@@ -875,7 +877,7 @@ START_TEST(test_g_tree_merge_and)
 		tree_add_key(b, r);
 
 	g_tree_merge(a,b,IST_SUBSEARCH_AND);
-	fail_unless(g_tree_nnodes(a)==45001,"g_tree_merge failed. Too few nodes in a. [%ld]", g_tree_nnodes(a));
+	fail_unless(g_tree_nnodes(a)==45001,"g_tree_merge failed. Too few nodes in a. [%d]", g_tree_nnodes(a));
 	
 	g_tree_destroy(a);
 	g_tree_destroy(b);
@@ -1025,7 +1027,7 @@ int main(void)
 	int nf;
 	Suite *s = dbmail_deliver_suite();
 	SRunner *sr = srunner_create(s);
-	g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
+	g_mime_init();
 	srunner_run_all(sr, CK_NORMAL);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
