@@ -1,5 +1,7 @@
 /*
- *  Copyright (c) 2004-2012 NFG Net Facilities Group BV support@nfg.nl
+ *   Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
+ *   Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
+ *   Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public License
@@ -629,7 +631,7 @@ int main(void)
 	int nf;
 	Suite *s = dbmail_mailbox_suite();
 	SRunner *sr = srunner_create(s);
-	g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
+	g_mime_init();
 	srunner_run_all(sr, CK_NORMAL);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
